#include "const.h"
#include <iostream>
#include <string>
#include <thread>
#include <wolfssl/options.h>
#include <wolfssl/ssl.h>

WOLFSSL_CTX *create_context(bool is_server) {
  WOLFSSL_METHOD *method =
      is_server ? wolfTLSv1_3_server_method() : wolfTLSv1_3_client_method();

  WOLFSSL_CTX *ctx = wolfSSL_CTX_new(method);
  if (ctx == NULL) {
    std::cerr << "Failed to create wolfSSL context" << std::endl;
    return nullptr;
  }

  // Set minimum protocol version to TLS 1.3
  wolfSSL_CTX_set_min_proto_version(ctx, WOLFSSL_TLSV1_3);

  if (is_server) {
    if (wolfSSL_CTX_use_certificate_file(ctx, CERTS_DIR "server.crt",
                                         WOLFSSL_FILETYPE_PEM) !=
        WOLFSSL_SUCCESS) {
      std::cerr << "Failed to load server certificate" << std::endl;
      wolfSSL_CTX_free(ctx);
      return nullptr;
    }

    if (wolfSSL_CTX_use_PrivateKey_file(ctx, CERTS_DIR "server.key",
                                        WOLFSSL_FILETYPE_PEM) !=
        WOLFSSL_SUCCESS) {
      unsigned long err = wolfSSL_ERR_get_error();
      char err_buf[256];
      wolfSSL_ERR_error_string(err, err_buf);
      std::cerr << "Failed to load server private key" << std::endl;
      std::cerr << "Error code: " << err << std::endl;
      std::cerr << "Error description: " << err_buf << std::endl;
      wolfSSL_CTX_free(ctx);
      return nullptr;
    }
  } else {
    if (wolfSSL_CTX_use_certificate_file(ctx, CERTS_DIR "client.crt",
                                         WOLFSSL_FILETYPE_PEM) !=
        WOLFSSL_SUCCESS) {
      std::cerr << "Failed to load client certificate" << std::endl;
    }

    if (wolfSSL_CTX_use_PrivateKey_file(ctx, CERTS_DIR "client.key",
                                        WOLFSSL_FILETYPE_PEM) !=
        WOLFSSL_SUCCESS) {
      std::cerr << "Failed to load client private key" << std::endl;
    }
  }

  // Enable peer verification
  wolfSSL_CTX_set_verify(ctx, WOLFSSL_VERIFY_PEER, nullptr);

  if (wolfSSL_CTX_load_verify_locations(ctx, CERTS_DIR "ca.crt", nullptr) !=
      WOLFSSL_SUCCESS) {
    std::cerr << "Failed to load CA certificate" << std::endl;
  }

  return ctx;
}

void handle_connection(WOLFSSL *ssl) {
  char buf[1024];
  std::cout << "Starting connection handler for wolfSSL session" << std::endl;

  while (true) {
    int bytes = wolfSSL_read(ssl, buf, sizeof(buf) - 1);
    if (bytes > 0) {
      buf[bytes] = '\0';
      std::cout << "Received " << bytes << " bytes from server: " << buf
                << std::endl;
    } else if (bytes == 0) {
      std::cout << "Connection closed by peer" << std::endl;
      break;
    } else {
      int ssl_error = wolfSSL_get_error(ssl, bytes);
      std::cerr << "wolfSSL_read failed with error: " << ssl_error << std::endl;
      break;
    }
  }
  std::cout << "Connection handler finished" << std::endl;
}
