#include "../common/cleanup.h"
#include "../common/const.h"
#include "../common/global.h"
#include "../common/tls.h"

#include "IF_functions.h"
#include "capture.h"

#include <cerrno>
#include <csignal>
#include <cstring>
#include <iostream>
#include <netdb.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <thread>
#include <unistd.h>
#include <wolfssl/options.h>
#include <wolfssl/ssl.h>

WOLFSSL_CTX *ctx;
int sockfd;

int main(int argc, char **argv) {
  if (argc < 2) {
    std::cerr << "Usage: " << argv[0] << " <interface_name>" << std::endl;
    return EINVAL;
  }
  std::signal(SIGTERM, safe_exit);
  std::signal(SIGINT, safe_exit);
  std::signal(SIGQUIT, safe_exit);
  std::signal(SIGHUP, safe_exit);

  // error signal handlers
  signal(SIGSEGV, safe_exit);

  // Initialize wolfSSL library
  wolfSSL_Init();

  ctx = create_context(true);

  sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if (sockfd < 0) {
    std::cerr << "Socket creation failed: " << strerror(errno) << std::endl;
    return 1;
  }

  std::cout << "Socket created successfully (fd: " << sockfd << ")"
            << std::endl;

  sockaddr_in addr{};
  addr.sin_family = AF_INET;
  addr.sin_port = htons(PORT);
  addr.sin_addr.s_addr = INADDR_ANY;

  if (bind(sockfd, (sockaddr *)&addr, sizeof(addr)) != 0) {
    std::cerr << "Bind failed: " << strerror(errno) << std::endl;
    return 1;
  }

  listen(sockfd, 1);

  while (true) {
    sockaddr_in client_addr{};
    socklen_t len = sizeof(client_addr);
    int client_fd = accept(sockfd, (sockaddr *)&client_addr, &len);

    WOLFSSL *ssl = wolfSSL_new(ctx);
    wolfSSL_set_fd(ssl, client_fd);

    if (wolfSSL_accept(ssl) != WOLFSSL_SUCCESS) {
      std::cerr << "wolfSSL_accept failed" << std::endl;
      wolfSSL_free(ssl);
      close(client_fd);
      continue;
    }

    { // send intreface ip to client
      std::string interface_ip = getInterfaceIP(argv[1]);
      if (interface_ip.empty()) {
        std::cerr << "Failed to get IP address for interface: " << argv[1]
                  << std::endl;
        wolfSSL_free(ssl);
        close(client_fd);
        continue;
      }

      interface_ip.insert(0, 1, IP_PACKET_TYPE);

      int bytes_sent =
          wolfSSL_write(ssl, interface_ip.c_str(), interface_ip.size());
      if (bytes_sent <= 0) {
        int ssl_error = wolfSSL_get_error(ssl, bytes_sent);
        std::cerr << "wolfSSL_write failed with error: " << ssl_error
                  << std::endl;
        wolfSSL_free(ssl);
        close(client_fd);
        continue;
      }
    }

    // handle_connection(ssl);
    std::thread IP_capture_thread(listen_for_ip_packets, argv[1], ssl);

    wolfSSL_shutdown(ssl);
    wolfSSL_free(ssl);
    close(client_fd);
  }

  wolfSSL_CTX_free(ctx);
  wolfSSL_Cleanup();
  close(sockfd);
  return 0;
}
